@@ -10,10 +10,10 @@ Adapted for numpy_core 2005 by Travis Oliphant and
 import types, sys
 
 import umath
-import oldnumeric
+import fromnumeric
 from numeric import newaxis, ndarray, inf
-from oldnumeric import typecodes, amax, amin
-from numerictypes import bool_
+from fromnumeric import amax, amin
+from numerictypes import bool_, typecodes
 import numeric
 import warnings
 
@@ -29,10 +29,14 @@ divide_tolerance = 1.e-35
 class MAError (Exception):
     def __init__ (self, args=None):
         "Create an exception"
+
+        # The .args attribute must be a tuple.
+        if not isinstance(args, tuple):
+            args = (args,)
         self.args = args
     def __str__(self):
         "Calculate the string representation"
-        return str(self.args)
+        return str(self.args[0])
     __repr__ = __str__
 
 class _MaskedPrintOption:
@@ -186,7 +190,7 @@ def make_mask (m, copy=0, flag=0):
     else:
         result = filled(m, True).astype(MaskType)
 
-    if flag and not oldnumeric.sometrue(oldnumeric.ravel(result)):
+    if flag and not fromnumeric.sometrue(fromnumeric.ravel(result)):
         return nomask
     else:
         return result
@@ -348,7 +352,7 @@ class domained_binary_operation:
         d2 = filled(b, self.filly)
         t = self.domain(d1, d2)
 
-        if oldnumeric.sometrue(t, None):
+        if fromnumeric.sometrue(t, None):
             d2 = where(t, self.filly, d2)
             mb = mask_or(mb, t)
         m = mask_or(ma, mb)
@@ -376,6 +380,10 @@ class masked_binary_operation:
         d1 = filled(a, self.fillx)
         d2 = filled(b, self.filly)
         result = self.f(d1, d2, *args, **kwargs)
+        if isinstance(result, ndarray) \
+               and m.ndim != 0 \
+               and m.shape != result.shape:
+            m = mask_or(getmaskarray(a), getmaskarray(b))
         return masked_array(result, m)
 
     def reduce (self, target, axis=0, dtype=None):
@@ -445,13 +453,12 @@ fabs = masked_unary_operation(umath.fabs)
 negative = masked_unary_operation(umath.negative)
 
 def nonzero(a):
-    """returns the indices of the elements of a which are not zero and not masked
-
-    a must be 1d
+    """returns the indices of the elements of a which are not zero
+    and not masked
     """
-    return asarray(filled(a, 0).nonzero())
+    return numeric.asarray(filled(a, 0).nonzero())
 
-around = masked_unary_operation(oldnumeric.round_)
+around = masked_unary_operation(fromnumeric.round_)
 floor = masked_unary_operation(umath.floor)
 ceil = masked_unary_operation(umath.ceil)
 logical_not = masked_unary_operation(umath.logical_not)
@@ -490,13 +497,13 @@ bitwise_or = masked_binary_operation(umath.bitwise_or)
 bitwise_xor = masked_binary_operation(umath.bitwise_xor)
 
 def rank (object):
-    return oldnumeric.rank(filled(object))
+    return fromnumeric.rank(filled(object))
 
 def shape (object):
-    return oldnumeric.shape(filled(object))
+    return fromnumeric.shape(filled(object))
 
 def size (object, axis=None):
-    return oldnumeric.size(filled(object), axis)
+    return fromnumeric.size(filled(object), axis)
 
 class MaskedArray (object):
     """Arrays with possibly masked values.
@@ -512,7 +519,7 @@ class MaskedArray (object):
            then the candidate data is data.data and the
            mask used is data.mask. If data is a numeric array,
            it is used as the candidate raw data.
-           If dtype.char is not None and
+           If dtype is not None and
            is != data.dtype.char then a data copy is required.
            Otherwise, the candidate is used.
 
@@ -580,10 +587,10 @@ class MaskedArray (object):
                 nd = size(self._data)
                 if nm != nd:
                     if nm == 1:
-                        self._mask = oldnumeric.resize(self._mask, self._data.shape)
+                        self._mask = fromnumeric.resize(self._mask, self._data.shape)
                         self._shared_mask = 0
                     elif nd == 1:
-                        self._data = oldnumeric.resize(self._data, self._mask.shape)
+                        self._data = fromnumeric.resize(self._data, self._mask.shape)
                         self._data.shape = self._mask.shape
                     else:
                         raise MAError, "Mask and data not compatible."
@@ -596,7 +603,7 @@ class MaskedArray (object):
     def __array__ (self, t=None, context=None):
         "Special hook for numeric. Converts to numeric if possible."
         if self._mask is not nomask:
-            if oldnumeric.ravel(self._mask).any():
+            if fromnumeric.ravel(self._mask).any():
                 if context is None:
                     warnings.warn("Cannot automatically convert masked array to "\
                                   "numeric because data\n    is masked in one or "\
@@ -621,12 +628,14 @@ class MaskedArray (object):
         else:
             return self._data
 
-    def __array_wrap__ (self, array, context):
+    def __array_wrap__ (self, array, context=None):
         """Special hook for ufuncs.
 
         Wraps the numpy array and sets the mask according to
         context.
         """
+        if context is None:
+            return MaskedArray(array, copy=False, mask=nomask)
         func, args = context[:2]
         domain = ufunc_domain[func]
         m = reduce(mask_or, [getmask(a) for a in args])
@@ -677,7 +686,7 @@ class MaskedArray (object):
             return masked_array(self._data.real, mask=nomask,
                             fill_value = self.fill_value())
         else:
-            return masked_array(self._data.real, mask=self._mask.ravel(),
+            return masked_array(self._data.real, mask=self._mask,
                             fill_value = self.fill_value())
 
     def _set_real (self, value):
@@ -691,7 +700,7 @@ class MaskedArray (object):
             return masked_array(self._data.imag, mask=nomask,
                             fill_value = self.fill_value())
         else:
-            return masked_array(self._data.imag, mask=self._mask.ravel(),
+            return masked_array(self._data.imag, mask=self._mask,
                             fill_value = self.fill_value())
 
     def _set_imaginary (self, value):
@@ -788,7 +797,13 @@ array(data = %(data)s,
         m = self._mask
         dout = self._data[i]
         if m is nomask:
-            return dout
+            try:
+                if dout.size == 1:
+                    return dout
+                else:
+                    return masked_array(dout, fill_value=self._fill_value)
+            except AttributeError:
+                return dout
         mi = m[i]
         if mi.size == 1:
             if mi:
@@ -798,16 +813,6 @@ array(data = %(data)s,
         else:
             return masked_array(dout, mi, fill_value=self._fill_value)
 
-    def __getslice__(self, i, j):
-        "Get slice described by i, j"
-        self.unshare_mask()
-        m = self._mask
-        dout = self._data[i:j]
-        if m is nomask:
-            return masked_array(dout, fill_value=self._fill_value)
-        else:
-            return masked_array(dout, mask = m[i:j], fill_value=self._fill_value)
-
 # --------
 # setitem and setslice notes
 # note that if value is masked, it means to mask those locations.
@@ -817,7 +822,7 @@ array(data = %(data)s,
         "Set item described by index. If value is masked, mask those locations."
         d = self._data
         if self is masked:
-            raise MAError, 'Cannot alter the masked element.'
+            raise MAError, 'Cannot alter masked elements.'
         if value is masked:
             if self._mask is nomask:
                 self._mask = make_mask_none(d.shape)
@@ -841,30 +846,6 @@ array(data = %(data)s,
                 self.unshare_mask()
             self._mask[index] = m
 
-    def __setslice__(self, i, j, value):
-        "Set slice i:j; if value is masked, mask those locations."
-        d = self._data
-        if self is masked:
-            raise MAError, "Cannot alter the 'masked' object."
-        if value is masked:
-            if self._mask is nomask:
-                self._mask = make_mask_none(d.shape)
-                self._shared_mask = False
-            self._mask[i:j] = True
-            return
-        m = getmask(value)
-        value = filled(value).astype(d.dtype)
-        d[i:j] = value
-        if m is nomask:
-            if self._mask is not nomask:
-                self.unshare_mask()
-                self._mask[i:j] = False
-        else:
-            if self._mask is nomask:
-                self._mask = make_mask_none(self._data.shape)
-                self._shared_mask = False
-            self._mask[i:j] = m
-
     def __nonzero__(self):
         """returns true if any element is non-zero or masked
 
@@ -1177,12 +1158,12 @@ array(data = %(data)s,
 
     def compressed (self):
         "A 1-D array of all the non-masked data."
-        d = oldnumeric.ravel(self._data)
+        d = fromnumeric.ravel(self._data)
         if self._mask is nomask:
             return array(d)
         else:
-            m = 1 - oldnumeric.ravel(self._mask)
-            c = oldnumeric.compress(m, d)
+            m = 1 - fromnumeric.ravel(self._mask)
+            c = fromnumeric.compress(m, d)
             return array(c, copy=0)
 
     def count (self, axis = None):
@@ -1203,7 +1184,7 @@ array(data = %(data)s,
                 del t[axis]
                 return ones(t) * n
         if axis is None:
-            w = oldnumeric.ravel(m).astype(int)
+            w = fromnumeric.ravel(m).astype(int)
             n1 = size(w)
             if n1 == 1:
                 n2 = w[0]
@@ -1254,15 +1235,7 @@ array(data = %(data)s,
                     #ok, can't put that value in here
                     value = numeric.array(value, dtype=object)
                     d = d.astype(object)
-                    result = oldnumeric.choose(m, (d, value))
-                except IndexError:
-                    #ok, if scalar
-                    if d.shape:
-                        raise
-                    elif m:
-                        result = numeric.array(value, dtype=d.dtype)
-                    else:
-                        result = d
+                    result = fromnumeric.choose(m, (d, value))
             return result
 
     def ids (self):
@@ -1291,7 +1264,7 @@ array(data = %(data)s,
         if self._mask is nomask:
             ind = iota
         else:
-            ind = oldnumeric.compress(1 - self._mask, iota)
+            ind = fromnumeric.compress(1 - self._mask, iota)
         d[ind] =  filled(values).astype(d.dtype)
 
     def putmask (self, values):
@@ -1355,16 +1328,22 @@ array(data = %(data)s,
         return self._data.dtype
     dtype = property(fget=_get_dtype, doc="type of the array elements.")
 
-    def item(self):
-        "Return Python scalar if possible."
+    def item(self, *args):
+        "Return Python scalar if possible"
         if self._mask is not nomask:
-            m = oldnumeric.ravel(self._mask)
+            m = self._mask.item(*args)
             try:
                 if m[0]:
                     return masked
             except IndexError:
                 return masked
-        return self._data.item()
+        return self._data.item(*args)
+
+    def itemset(self, *args):
+        "Set Python scalar into array"
+        item = args[-1]
+        args = args[:-1]
+        self[args] = item
 
     def tolist(self, fill_value=None):
         "Convert to list"
@@ -1388,6 +1367,14 @@ array(data = %(data)s,
             self._mask = make_mask (self._mask, copy=1, flag=0)
             self._shared_mask = 0
 
+    def _get_ctypes(self):
+        return self._data.ctypes
+
+    def _get_T(self):
+        if (self.ndim < 2):
+            return self
+        return self.transpose()
+
     shape = property(_get_shape, _set_shape,
            doc = 'tuple giving the shape of the array')
 
@@ -1402,6 +1389,10 @@ array(data = %(data)s,
 
     imag = imaginary
 
+    ctypes = property(_get_ctypes, None, doc="ctypes")
+
+    T = property(_get_T, None, doc="get transpose")
+
 #end class MaskedArray
 
 array = MaskedArray
@@ -1428,7 +1419,7 @@ def allclose (a, b, fill_value=1, rtol=1.e-5, atol=1.e-8):
     x = filled(array(d1, copy=0, mask=m), fill_value).astype(float)
     y = filled(array(d2, copy=0, mask=m), 1).astype(float)
     d = umath.less_equal(umath.absolute(x-y), atol + rtol * umath.absolute(y))
-    return oldnumeric.alltrue(oldnumeric.ravel(d))
+    return fromnumeric.alltrue(fromnumeric.ravel(d))
 
 def allequal (a, b, fill_value=1):
     """
@@ -1440,13 +1431,13 @@ def allequal (a, b, fill_value=1):
         x = filled(a)
         y = filled(b)
         d = umath.equal(x, y)
-        return oldnumeric.alltrue(oldnumeric.ravel(d))
+        return fromnumeric.alltrue(fromnumeric.ravel(d))
     elif fill_value:
         x = filled(a)
         y = filled(b)
         d = umath.equal(x, y)
         dm = array(d, mask=m, copy=0)
-        return oldnumeric.alltrue(oldnumeric.ravel(filled(dm, 1)))
+        return fromnumeric.alltrue(fromnumeric.ravel(filled(dm, 1)))
     else:
         return 0
 
@@ -1477,13 +1468,13 @@ def masked_object (data, value, copy=1):
     dm = make_mask(umath.equal(d, value), flag=1)
     return array(d, mask=dm, copy=copy, fill_value=value)
 
-def arrayrange(start, stop=None, step=1, dtype=None):
+def arange(start, stop=None, step=1, dtype=None):
     """Just like range() except it returns a array whose type can be specified
     by the keyword argument dtype.
     """
-    return array(numeric.arrayrange(start, stop, step, dtype))
+    return array(numeric.arange(start, stop, step, dtype))
 
-arange = arrayrange
+arrayrange = arange
 
 def fromstring (s, t):
     "Construct a masked array from a string. Result will have no mask."
@@ -1514,24 +1505,28 @@ def resize (a, new_shape):
     The original array's total size can be any size."""
     m = getmask(a)
     if m is not nomask:
-        m = oldnumeric.resize(m, new_shape)
-    result = array(oldnumeric.resize(filled(a), new_shape), mask=m)
+        m = fromnumeric.resize(m, new_shape)
+    result = array(fromnumeric.resize(filled(a), new_shape), mask=m)
     result.set_fill_value(get_fill_value(a))
     return result
 
-def repeat(a, repeats, axis=0):
+def repeat(a, repeats, axis=None):
     """repeat elements of a repeats times along axis
        repeats is a sequence of length a.shape[axis]
        telling how many times to repeat each element.
     """
     af = filled(a)
     if isinstance(repeats, types.IntType):
-        repeats = tuple([repeats]*(shape(af)[axis]))
+        if axis is None:
+            num = af.size
+        else:
+            num = af.shape[axis]
+        repeats = tuple([repeats]*num)
 
     m = getmask(a)
     if m is not nomask:
-        m = oldnumeric.repeat(m, repeats, axis)
-    d = oldnumeric.repeat(af, repeats, axis)
+        m = fromnumeric.repeat(m, repeats, axis)
+    d = fromnumeric.repeat(af, repeats, axis)
     result = masked_array(d, m)
     result.set_fill_value(get_fill_value(a))
     return result
@@ -1547,17 +1542,16 @@ def indices (dimensions, dtype=None):
     """
     return array(numeric.indices(dimensions, dtype))
 
-def zeros (shape, dtype=int):
-    """zeros(n, dtype=int) =
+def zeros (shape, dtype=float):
+    """zeros(n, dtype=float) =
      an array of all zeros of the given length or shape."""
     return array(numeric.zeros(shape, dtype))
 
-def ones (shape, dtype=int):
-    """ones(n, dtype=int) =
+def ones (shape, dtype=float):
+    """ones(n, dtype=float) =
      an array of all ones of the given length or shape."""
     return array(numeric.ones(shape, dtype))
 
-
 def count (a, axis = None):
     "Count of the non-masked elements in a, or along a certain axis."
     a = masked_array(a)
@@ -1574,7 +1568,7 @@ def power (a, b, third=None):
     fb = filled(b, 1)
     if fb.dtype.char in typecodes["Integer"]:
         return masked_array(umath.power(fa, fb), m)
-    md = make_mask(umath.less_equal (fa, 0), flag=1)
+    md = make_mask(umath.less(fa, 0), flag=1)
     m = mask_or(m, md)
     if m is nomask:
         return masked_array(umath.power(fa, fb))
@@ -1588,16 +1582,25 @@ def masked_array (a, mask=nomask, fill_value=None):
     """
     return array(a, mask=mask, copy=0, fill_value=fill_value)
 
-sum = add.reduce
-product = multiply.reduce
+def sum (target, axis=None, dtype=None):
+    if axis is None:
+        target = ravel(target)
+        axis = 0
+    return add.reduce(target, axis, dtype)
 
-def average (a, axis=0, weights=None, returned = 0):
-    """average(a, axis=0, weights=None)
+def product (target, axis=None, dtype=None):
+    if axis is None:
+        target = ravel(target)
+        axis = 0
+    return multiply.reduce(target, axis, dtype)
+
+def average (a, axis=None, weights=None, returned = 0):
+    """average(a, axis=None, weights=None)
        Computes average along indicated axis.
        If axis is None, average over the entire array
        Inputs can be integer or floating types; result is of type float.
 
-       If weights are given, result is sum(a*weights)/(sum(weights)*1.0)
+       If weights are given, result is sum(a*weights,axis=0)/(sum(weights,axis=0)*1.0)
        weights must have a's shape or be the 1-d with length the size
        of a in the given axis.
 
@@ -1624,7 +1627,7 @@ def average (a, axis=0, weights=None, returned = 0):
         else:
             if weights is None:
                 n = add.reduce(a.ravel())
-                w = oldnumeric.choose(mask, (1.0, 0.0)).ravel()
+                w = fromnumeric.choose(mask, (1.0, 0.0)).ravel()
                 d = umath.add.reduce(w)
                 del w
             else:
@@ -1648,7 +1651,6 @@ def average (a, axis=0, weights=None, returned = 0):
                     d = add.reduce(w, axis)
                     del w
                 elif wsh == (ash[axis],):
-                    ni = ash[axis]
                     r = [newaxis]*len(ash)
                     r[axis] = slice(None, None, 1)
                     w = eval ("w["+ repr(tuple(r)) + "] * ones(ash, float)")
@@ -1673,7 +1675,6 @@ def average (a, axis=0, weights=None, returned = 0):
                     n = add.reduce(a*w, axis)
                     d = add.reduce(w, axis)
                 elif wsh == (ash[axis],):
-                    ni = ash[axis]
                     r = [newaxis]*len(ash)
                     r[axis] = slice(None, None, 1)
                     w = eval ("w["+ repr(tuple(r)) + "] * masked_array(ones(ash, float), mask)")
@@ -1718,7 +1719,7 @@ def where (condition, x, y):
     m = make_mask(mask_or(m, md), copy=0, flag=1)
     return masked_array(d, m)
 
-def choose (indices, t):
+def choose (indices, t, out=None, mode='raise'):
     "Returns array shaped like indices with elements chosen from t"
     def fmask (x):
         if x is masked: return 1
@@ -1814,7 +1815,7 @@ def reshape (a, *newshape):
 def ravel (a):
     "a as one-dimensional, may share data and mask"
     m = getmask(a)
-    d = oldnumeric.ravel(filled(a))
+    d = fromnumeric.ravel(filled(a))
     if m is nomask:
         return masked_array(d)
     else:
@@ -1846,8 +1847,8 @@ def swapaxes (a, axis1, axis2):
                             mask=numeric.swapaxes(m, axis1, axis2),)
 
 
-def take (a, indices, axis=0):
-    "take(a, indices, axis=0) returns selection of items from a."
+def take (a, indices, axis=None, out=None, mode='raise'):
+    "returns selection of items from a."
     m = getmask(a)
     # d = masked_array(a).raw_data()
     d = masked_array(a).data
@@ -1858,7 +1859,7 @@ def take (a, indices, axis=0):
                      mask = numeric.take(m, indices, axis))
 
 def transpose(a, axes=None):
-    "transpose(a, axes=None) reorder dimensions per tuple axes"
+    "reorder dimensions per tuple axes"
     m = getmask(a)
     d = filled(a)
     if m is nomask:
@@ -1868,8 +1869,8 @@ def transpose(a, axes=None):
                      mask = numeric.transpose(m, axes))
 
 
-def put(a, indices, values):
-    """put(a, indices, values) sets storage-indexed locations to corresponding values.
+def put(a, indices, values, mode='raise'):
+    """sets storage-indexed locations to corresponding values.
 
     Values and indices are filled if necessary.
 
@@ -1893,8 +1894,8 @@ def putmask(a, mask, values):
     a.unshare_mask()
     numeric.putmask(a.raw_mask(), mask, 0)
 
-def innerproduct(a, b):
-    """innerproduct(a,b) returns the dot product of two arrays, which has
+def inner(a, b):
+    """inner(a,b) returns the dot product of two arrays, which has
     shape a.shape[:-1] + b.shape[:-1] with elements computed by summing the
     product of the elements from the last dimensions of a and b.
     Masked elements are replace by zeros.
@@ -1903,22 +1904,26 @@ def innerproduct(a, b):
     fb = filled(b, 0)
     if len(fa.shape) == 0: fa.shape = (1,)
     if len(fb.shape) == 0: fb.shape = (1,)
-    return masked_array(numeric.innerproduct(fa, fb))
+    return masked_array(numeric.inner(fa, fb))
 
-def outerproduct(a, b):
-    """outerproduct(a,b) = {a[i]*b[j]}, has shape (len(a),len(b))"""
+innerproduct = inner
+
+def outer(a, b):
+    """outer(a,b) = {a[i]*b[j]}, has shape (len(a),len(b))"""
     fa = filled(a, 0).ravel()
     fb = filled(b, 0).ravel()
-    d = numeric.outerproduct(fa, fb)
+    d = numeric.outer(fa, fb)
     ma = getmask(a)
     mb = getmask(b)
     if ma is nomask and mb is nomask:
         return masked_array(d)
     ma = getmaskarray(a)
     mb = getmaskarray(b)
-    m = make_mask(1-numeric.outerproduct(1-ma, 1-mb), copy=0)
+    m = make_mask(1-numeric.outer(1-ma, 1-mb), copy=0)
     return masked_array(d, m)
 
+outerproduct = outer
+
 def dot(a, b):
     """dot(a,b) returns matrix-multiplication between a and b.  The product-sum
     is over the last dimension of a and the second-to-last dimension of b.
@@ -1926,7 +1931,7 @@ def dot(a, b):
     """
     return innerproduct(filled(a, 0), numeric.swapaxes(filled(b, 0), -1, -2))
 
-def compress(condition, x, dimension=-1):
+def compress(condition, x, dimension=-1, out=None):
     """Select those parts of x for which condition is true.
        Masked values in condition are considered false.
     """
@@ -2049,36 +2054,36 @@ def sort (x, axis = -1, fill_value=None):
     if fill_value is None:
         fill_value = default_fill_value (x)
     d = filled(x, fill_value)
-    s = oldnumeric.sort(d, axis)
+    s = fromnumeric.sort(d, axis)
     if getmask(x) is nomask:
         return masked_array(s)
     return masked_values(s, fill_value, copy=0)
 
 def diagonal(a, k = 0, axis1=0, axis2=1):
     """diagonal(a,k=0,axis1=0, axis2=1) = the k'th diagonal of a"""
-    d = oldnumeric.diagonal(filled(a), k, axis1, axis2)
+    d = fromnumeric.diagonal(filled(a), k, axis1, axis2)
     m = getmask(a)
     if m is nomask:
         return masked_array(d, m)
     else:
-        return masked_array(d, oldnumeric.diagonal(m, k, axis1, axis2))
-    
-def trace (a, offset=0, axis1=0, axis2=1, dtype=None):
+        return masked_array(d, fromnumeric.diagonal(m, k, axis1, axis2))
+
+def trace (a, offset=0, axis1=0, axis2=1, dtype=None, out=None):
     """trace(a,offset=0, axis1=0, axis2=1) returns the sum along diagonals
     (defined by the last two dimenions) of the array.
     """
     return diagonal(a, offset, axis1, axis2).sum(dtype=dtype)
 
-def argsort (x, axis = -1, fill_value=None):
+def argsort (x, axis = -1, out=None, fill_value=None):
     """Treating masked values as if they have the value fill_value,
        return sort indices for sorting along given axis.
        if fill_value is None, use get_fill_value(x)
        Returns a numpy array.
     """
     d = filled(x, fill_value)
-    return oldnumeric.argsort(d, axis)
+    return fromnumeric.argsort(d, axis)
 
-def argmin (x, axis = -1, fill_value=None):
+def argmin (x, axis = -1, out=None, fill_value=None):
     """Treating masked values as if they have the value fill_value,
        return indices for minimum values along given axis.
        if fill_value is None, use get_fill_value(x).
@@ -2086,9 +2091,9 @@ def argmin (x, axis = -1, fill_value=None):
        Otherwise, returns a scalar index.
     """
     d = filled(x, fill_value)
-    return oldnumeric.argmin(d, axis)
+    return fromnumeric.argmin(d, axis)
 
-def argmax (x, axis = -1, fill_value=None):
+def argmax (x, axis = -1, out=None, fill_value=None):
     """Treating masked values as if they have the value fill_value,
        return sort indices for maximum along given axis.
        if fill_value is None, use -get_fill_value(x) if it exists.
@@ -2102,7 +2107,7 @@ def argmax (x, axis = -1, fill_value=None):
         except:
             pass
     d = filled(x, fill_value)
-    return oldnumeric.argmax(d, axis)
+    return fromnumeric.argmax(d, axis)
 
 def fromfunction (f, s):
     """apply f to s to create array as in umath."""
@@ -2132,19 +2137,19 @@ array.argsort = _m(argsort)
 array.base = property(_m(not_implemented))
 array.byteswap = _m(not_implemented)
 
-def _choose(self, *args):
+def _choose(self, *args, **kwds):
     return choose(self, args)
 array.choose = _m(_choose)
 del _choose
 
-def _clip(self,a_min,a_max):
-    return MaskedArray(data = self.data.clip(asarray(a_min).data, 
+def _clip(self,a_min,a_max,out=None):
+    return MaskedArray(data = self.data.clip(asarray(a_min).data,
                                              asarray(a_max).data),
                        mask = mask_or(self.mask,
                                       mask_or(getmask(a_min),getmask(a_max))))
 array.clip = _m(_clip)
 
-def _compress(self, cond, axis=None):
+def _compress(self, cond, axis=None, out=None):
     return compress(cond, self, axis)
 array.compress = _m(_compress)
 del _compress
@@ -2152,14 +2157,14 @@ del _compress
 array.conj = array.conjugate = _m(conjugate)
 array.copy = _m(not_implemented)
 
-def _cumprod(self, axis=0, dtype=None):
+def _cumprod(self, axis=None, dtype=None, out=None):
     m = self.mask
     if m is not nomask:
         m = umath.logical_or.accumulate(self.mask, axis)
     return MaskedArray(data = self.filled(1).cumprod(axis, dtype), mask=m)
 array.cumprod = _m(_cumprod)
 
-def _cumsum(self, axis=0, dtype=None):
+def _cumsum(self, axis=None, dtype=None, out=None):
     m = self.mask
     if m is not nomask:
         m = umath.logical_or.accumulate(self.mask, axis)
@@ -2174,14 +2179,18 @@ array.flags = property(_m(not_implemented))
 array.flatten = _m(ravel)
 array.getfield = _m(not_implemented)
 
-def _max(a, axis=None):
+def _max(a, axis=None, out=None):
+    if out is not None:
+        raise TypeError("Output arrays Unsupported for masked arrays")
     if axis is None:
         return maximum(a)
     else:
         return maximum.reduce(a, axis)
 array.max = _m(_max)
 del _max
-def _min(a, axis=None):
+def _min(a, axis=None, out=None):
+    if out is not None:
+        raise TypeError("Output arrays Unsupported for masked arrays")
     if axis is None:
         return minimum(a)
     else:
@@ -2194,8 +2203,8 @@ array.newbyteorder = _m(not_implemented)
 array.nonzero = _m(nonzero)
 array.prod = _m(product)
 
-def _ptp(a,axis=0):
-    return a.max(axis)-a.min(axis)
+def _ptp(a,axis=None,out=None):
+    return a.max(axis,out)-a.min(axis)
 array.ptp = _m(_ptp)
 array.repeat = _m(repeat)
 array.resize = _m(resize)
@@ -2211,7 +2220,7 @@ def _squeeze(self):
     except AttributeError:
         result = _wrapit(self, 'squeeze')
     return result
-array.squeeze = _m(_squeeze)      
+array.squeeze = _m(_squeeze)
 
 array.strides = property(_m(not_implemented))
 array.sum = _m(sum)
@@ -2224,15 +2233,15 @@ array.tofile = _m(not_implemented)
 array.trace = _m(trace)
 array.transpose = _m(transpose)
 
-def _var(self,axis=0,dtype=None):
+def _var(self,axis=None,dtype=None, out=None):
     if axis is None:
-        return asarray(self.compressed()).var()
+        return numeric.asarray(self.compressed()).var()
     a = self.swapaxes(axis,0)
     a = a - a.mean(axis=0)
     a *= a
-    a /= (a.count(axis=0)-1)
+    a /= a.count(axis=0)
     return a.swapaxes(0,axis).sum(axis)
-def _std(self,axis=0,dtype=None):
+def _std(self,axis=None, dtype=None, out=None):
     return (self.var(axis,dtype))**0.5
 array.var = _m(_var)
 array.std = _m(_std)
