@@ -10,8 +10,7 @@ __all__ = ['FCompiler','new_fcompiler','show_fcompilers',
 import os
 import sys
 import re
-from types import StringType,NoneType
-from distutils.sysconfig import get_config_var
+from distutils.sysconfig import get_config_var, get_python_lib
 from distutils.fancy_getopt import FancyGetopt
 from distutils.errors import DistutilsModuleError,DistutilsArgError,\
      DistutilsExecError,CompileError,LinkError,DistutilsPlatformError
@@ -29,7 +28,7 @@ class FCompiler(CCompiler):
 
     Methods that subclasses may redefine:
 
-        get_version_cmd(), get_linker_so(), get_version()
+        find_executables(), get_version_cmd(), get_linker_so(), get_version()
         get_flags(), get_flags_opt(), get_flags_arch(), get_flags_debug()
         get_flags_f77(), get_flags_opt_f77(), get_flags_arch_f77(),
         get_flags_debug_f77(), get_flags_f90(), get_flags_opt_f90(),
@@ -104,11 +103,21 @@ class FCompiler(CCompiler):
     shared_lib_format = "%s%s"
     exe_extension = ""
 
+    # If compiler does not support compiling Fortran 90 then it can
+    # suggest using another compiler. For example, gnu would suggest
+    # gnu95 compiler type when there are F90 sources.
+    suggested_f90_compiler = None
+
     ######################################################################
     ## Methods that subclasses may redefine. But don't call these methods!
     ## They are private to FCompiler class and may return unexpected
     ## results if used elsewhere. So, you have been warned..
 
+    def find_executables(self):
+        """Modify self.executables to hold found executables, instead of
+        searching for them at class creation time."""
+        pass
+
     def get_version_cmd(self):
         """ Compiler command to print out version information. """
         f77 = self.executables['compiler_f77']
@@ -263,6 +272,7 @@ class FCompiler(CCompiler):
             noarch = conf.get('noarch',[None,noopt])[1]
         debug = conf.get('debug',[None,0])[1]
 
+        self.find_executables()
 
         f77 = self.__get_cmd('compiler_f77','F77',(conf,'f77exec'))
         f90 = self.__get_cmd('compiler_f90','F90',(conf,'f90exec'))
@@ -333,7 +343,14 @@ class FCompiler(CCompiler):
         linker_so = self.__get_cmd(self.get_linker_so,'LDSHARED')
         if linker_so:
             linker_so_flags = self.__get_flags(self.get_flags_linker_so,'LDFLAGS')
-            self.set_executables(linker_so=[linker_so]+linker_so_flags)
+            if sys.platform.startswith('aix'):
+                python_lib = get_python_lib(standard_lib=1)
+                ld_so_aix = os.path.join(python_lib, 'config', 'ld_so_aix')
+                python_exp = os.path.join(python_lib, 'config', 'python.exp')
+                linker_so = [ld_so_aix, linker_so, '-bI:'+python_exp]
+            else:
+                linker_so = [linker_so]
+            self.set_executables(linker_so=linker_so+linker_so_flags)
 
         linker_exe = self.__get_cmd(self.get_linker_exe,'LD')
         if linker_exe:
@@ -380,21 +397,23 @@ class FCompiler(CCompiler):
 
     def _compile(self, obj, src, ext, cc_args, extra_postargs, pp_opts):
         """Compile 'src' to product 'obj'."""
+        src_flags = {}
         if is_f_file(src) and not has_f90_header(src):
             flavor = ':f77'
             compiler = self.compiler_f77
+            src_flags = get_f77flags(src)
         elif is_free_format(src):
             flavor = ':f90'
             compiler = self.compiler_f90
             if compiler is None:
-                raise DistutilsExecError, 'f90 not supported by '\
-                      +self.__class__.__name__
+                raise DistutilsExecError, 'f90 not supported by %s needed for %s'\
+                      % (self.__class__.__name__,src)
         else:
             flavor = ':fix'
             compiler = self.compiler_fix
             if compiler is None:
-                raise DistutilsExecError, 'f90 (fixed) not supported by '\
-                      +self.__class__.__name__
+                raise DistutilsExecError, 'f90 (fixed) not supported by %s needed for %s'\
+                      % (self.__class__.__name__,src)
         if self.object_switch[-1]==' ':
             o_args = [self.object_switch.strip(),obj]
         else:
@@ -403,9 +422,14 @@ class FCompiler(CCompiler):
         assert self.compile_switch.strip()
         s_args = [self.compile_switch, src]
 
+        extra_flags = src_flags.get(self.compiler_type,[])
+        if extra_flags:
+            log.info('using compile options from source: %r' \
+                     % ' '.join(extra_flags))
+
         if os.name == 'nt':
             compiler = _nt_quote_args(compiler)
-        command = compiler + cc_args + s_args + o_args + extra_postargs
+        command = compiler + cc_args + extra_flags + s_args + o_args + extra_postargs
 
         display = '%s: %s' % (os.path.basename(compiler[0]) + flavor,
                               src)
@@ -528,7 +552,7 @@ fcompiler_class = {'gnu':('gnu','GnuFCompiler',
                    'gnu95':('gnu','Gnu95FCompiler',
                             "GNU 95 Fortran Compiler"),
                    'g95':('g95','G95FCompiler',
-                          "GNU Fortran 95 Compiler"),
+                          "G95 Fortran Compiler"),
                    'pg':('pg','PGroupFCompiler',
                          "Portland Group Fortran Compiler"),
                    'absoft':('absoft','AbsoftFCompiler',
@@ -582,13 +606,26 @@ _default_compilers = (
     ('mac',('gnu','gnu95',)),
     )
 
-def _find_existing_fcompiler(compilers, osname=None, platform=None):
+def _find_existing_fcompiler(compilers, osname=None, platform=None, requiref90=None):
     for compiler in compilers:
         v = None
         try:
             c = new_fcompiler(plat=platform, compiler=compiler)
             c.customize()
             v = c.get_version()
+            if requiref90 and c.compiler_f90 is None:
+                v = None
+                new_compiler = c.suggested_f90_compiler
+                if new_compiler:
+                    log.warn('Trying %r compiler as suggested by %r compiler for f90 support.' % (compiler, new_compiler))
+                    c = new_fcompiler(plat=platform, compiler=new_compiler)
+                    c.customize()
+                    v = c.get_version()
+                    if v is not None:
+                        compiler = new_compiler
+            if requiref90 and c.compiler_f90 is None:
+                raise ValueError,'%s does not support compiling f90 codes, skipping.' \
+                      % (c.__class__.__name__)
         except DistutilsModuleError:
             pass
         except Exception, msg:
@@ -597,7 +634,7 @@ def _find_existing_fcompiler(compilers, osname=None, platform=None):
             return compiler
     return
 
-def get_default_fcompiler(osname=None, platform=None):
+def get_default_fcompiler(osname=None, platform=None, requiref90=None):
     """ Determine the default Fortran compiler to use for the given platform. """
     if osname is None:
         osname = os.name
@@ -615,7 +652,8 @@ def get_default_fcompiler(osname=None, platform=None):
         matching_compilers.append('gnu')
     compiler =  _find_existing_fcompiler(matching_compilers,
                                          osname=osname,
-                                         platform=platform)
+                                         platform=platform,
+                                         requiref90=requiref90)
     if compiler is not None:
         return compiler
     return matching_compilers[0]
@@ -624,7 +662,8 @@ def new_fcompiler(plat=None,
                   compiler=None,
                   verbose=0,
                   dry_run=0,
-                  force=0):
+                  force=0,
+                  requiref90=0):
     """ Generate an instance of some FCompiler subclass for the supplied
     platform/compiler combination.
     """
@@ -632,7 +671,7 @@ def new_fcompiler(plat=None,
         plat = os.name
     try:
         if compiler is None:
-            compiler = get_default_fcompiler(plat)
+            compiler = get_default_fcompiler(plat,requiref90=requiref90)
         (module_name, class_name, long_description) = fcompiler_class[compiler]
     except KeyError:
         msg = "don't know how to compile Fortran code on platform '%s'" % plat
@@ -671,12 +710,12 @@ def show_fcompilers(dist = None):
         dist.cmdclass['config_fc'] = config_fc
         dist.parse_config_files()
         dist.parse_command_line()
-
     compilers = []
     compilers_na = []
     compilers_ni = []
     for compiler in fcompiler_class.keys():
         v = 'N/A'
+        log.set_verbosity(-2)
         try:
             c = new_fcompiler(compiler=compiler)
             c.customize(dist)
@@ -685,6 +724,7 @@ def show_fcompilers(dist = None):
             pass
         except Exception, msg:
             log.warn(msg)
+
         if v is None:
             compilers_na.append(("fcompiler="+compiler, None,
                               fcompiler_class[compiler][2]))
@@ -728,7 +768,8 @@ is_f_file = re.compile(r'.*[.](for|ftn|f77|f)\Z',re.I).match
 _has_f_header = re.compile(r'-[*]-\s*fortran\s*-[*]-',re.I).search
 _has_f90_header = re.compile(r'-[*]-\s*f90\s*-[*]-',re.I).search
 _has_fix_header = re.compile(r'-[*]-\s*fix\s*-[*]-',re.I).search
-_free_f90_start = re.compile(r'[^c*]\s*[^\s\d\t]',re.I).match
+_free_f90_start = re.compile(r'[^c*!]\s*[^\s\d\t]',re.I).match
+
 def is_free_format(file):
     """Check if file is in free format Fortran."""
     # f90 allows both fixed and free format, assuming fixed unless
@@ -736,16 +777,17 @@ def is_free_format(file):
     result = 0
     f = open(file,'r')
     line = f.readline()
-    n = 15 # the number of non-comment lines to scan for hints
+    n = 10000 # the number of non-comment lines to scan for hints
     if _has_f_header(line):
         n = 0
     elif _has_f90_header(line):
         n = 0
         result = 1
     while n>0 and line:
-        if line[0]!='!':
+        line = line.rstrip()
+        if line and line[0]!='!':
             n -= 1
-            if (line[0]!='\t' and _free_f90_start(line[:5])) or line[-2:-1]=='&':
+            if (line[0]!='\t' and _free_f90_start(line[:5])) or line[-1:]=='&':
                 result = 1
                 break
         line = f.readline()
@@ -758,5 +800,26 @@ def has_f90_header(src):
     f.close()
     return _has_f90_header(line) or _has_fix_header(line)
 
+_f77flags_re = re.compile(r'(c|)f77flags\s*\(\s*(?P<fcname>\w+)\s*\)\s*=\s*(?P<fflags>.*)',re.I)
+def get_f77flags(src):
+    """
+    Search the first 20 lines of fortran 77 code for line pattern
+      `CF77FLAGS(<fcompiler type>)=<f77 flags>`
+    Return a dictionary {<fcompiler type>:<f77 flags>}.
+    """
+    flags = {}
+    f = open(src,'r')
+    i = 0
+    for line in f.readlines():
+        i += 1
+        if i>20: break
+        m = _f77flags_re.match(line)
+        if not m: continue
+        fcname = m.group('fcname').strip()
+        fflags = m.group('fflags').strip()
+        flags[fcname] = split_quoted(fflags)
+    f.close()
+    return flags
+
 if __name__ == '__main__':
     show_fcompilers()
